import jinro.*;
import scripts.*;
import scripts.roles.*;
import java.text.Collator;

class Game extends System {
	def strings = super.strings.attachBundle("Game.properties");
	// Argument time (minutes)
	def argumenttime = 7;
	// Night time (minutes)
	def nighttime = 3;
	// Limits the frequency of revotes per day.
	// If frequency of revotes exceeds the limit, the game ends in a draw.
	def revotelimit = 3;
	// Keeps roles/death secret from viewers if it's true.
	def issecret = false;
	def issecretsan = false;
	def initialSanity = 0;
	def sanPenaltyRate = 50;
	def sanPenaltyExecution = 3;
	def sanRecoveryExecution = 1;
	def sanPenaltyVictims = 1;
	def sanRecoveryVictims = 1;
	def sanPenaltyRandom = 2;
	def sanRecoveryRandom = 1;

	def bitetarget = null;
	def killedMessages = [];
	// Returns the game options.
	// This method is called by JinroGame.getOptionList().
	def getOptionList() {
		def list = [];
		
		list << [strings.getString("option.argumenttime"), "argumenttime", "Integer"];	//Argument time
		list << [strings.getString("option.nighttime"), "nighttime", "Integer"];		//Night time
		list << [strings.getString("option.revotelimit"), "revotelimit", "Integer"];	//Revote limit
		list << [strings.getString("option.issecret"), "issecret", "Boolean"];			//Secret mode
		list << [strings.getString("option.issecretsan"), "issecretsan", "Boolean"];
		list << [strings.getString("option.sanPenaltyRate"), "sanPenaltyRate", "Integer"];
		list << [strings.getString("option.sanPenaltyExecution"), "sanPenaltyExecution", "Integer"];
		list << [strings.getString("option.sanRecoveryExecution"), "sanRecoveryExecution", "Integer"];
		list << [strings.getString("option.sanPenaltyVictims"), "sanPenaltyVictims", "Integer"];
		list << [strings.getString("option.sanRecoveryVictims"), "sanRecoveryVictims", "Integer"];
		list << [strings.getString("option.sanPenaltyRandom"), "sanPenaltyRandom", "Integer"];
		list << [strings.getString("option.sanRecoveryRandom"), "sanRecoveryRandom", "Integer"];
		
		return list;
	}
	// Returns the actors data (for displaying).
	// This method is called by JinroGame.getActorsData().
	def getActorsData() {
		def list = [];
		actors.each { actor ->
			def data = [];
			data[1] = actor.name;
			// Hides roles and death from viewers if the game is secret mode.
			if(issecret && actor.state.priority < 50 && actor.state != State.escaped) {
				data[0] = State.bitten.priority;
				data[3] = strings.getString("secret.death");
			} else {
				data[0] = actor.state.priority;
				data[3] = actor.state.name;
			}
			if(issecret) {
				data[2] = strings.getString("secret.role");
				data[4] = strings.getString("secret.sanity");
				data[5] = 0;
			} else {
				data[2] = JinroGame.getAlias(actor.getRoleName());
				data[5] = actor.getPriority();
				if(issecretsan) {
					data[4] = strings.getString("secret.sanity");
				} else {
					if(actor.isSanityEnabled()) {
						data[4] = actor.sanity;
					} else {
						data[4] = strings.getString("sanity.none");
					}
				}
			}
			list << data;
		};
		// Sort the actors.
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		list.sort { one, two ->
			if(one[0] != two[0]) {
				// Actor's state
				return two[0] - one[0];
			} else if(one[5] != two[5]) {
				// Actor's role
				return two[5] - one[5];
			} else {
				// Actor's name
				return collator.compare(one[1], two[1]);
			}
		};
		return list;
	}
	// This method is called by JinroInterface.Update.CountDown.run().
	// If it returns true , the tool notifies all actors of the remaining time.
	//               false, you should write a code of notifying here.
	def timerCounted(minutes, seconds) {
		return true;
	}
	// Frequency of revotes
	def revoteCount = 0;
	// Calls when start argument/night time.
	// This method is called by JinroGame.changePhase().
	def startPhase() {
		if(judge()) {
			return;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				noticeall(strings.getString("notice.start.argument"));
				noticeall(strings.getString("notice.vote"));
				JinroInterface.setTimer(argumenttime, 0);
				JinroInterface.countdown = true;
				revoteCount = 0;
				break;
			case 1:
				noticeall(strings.getString("notice.start.night"));
				JinroInterface.setTimer(nighttime, 0);
				JinroInterface.countdown = true;
				break;
		}
		actors.findAll{ it.isLive() }.each {
			it.startPhase();
		}
	}
	// Calls when end argument/night time.
	// This method is called by JinroGame.changePhase().
	def endPhase() {
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				execution();
				break;
			case 1:
				if(JinroGame.day > 0) {
					if(!Wolf.bitetarget.defended) {
						Wolf.bitetarget.nextstate = State.bitten;
						Wolf.bitetarget.countdown = 1;
					}
				}
				break;
		}
		update();
		Wolf.bitetarget = null;
		return true;
	}
	// Checks if the game can go to next phase.
	// This method is called by JinroGame.checkStep().
	def checkStep() {
		def result = actors.find { it.isLive() && !it.checkStep() };
		if(result) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				if(!checkExecution()) {
					return false;
				}
				break;
			case 1:
				if(JinroGame.day > 0) {
					if(!Wolf.bitetarget) {
						return false;
					}
				}
				break;
		}
		return true;
	}
	// Checks if all actors finished vote.
	def checkExecution() {
		def lives = actors.findAll{ it.isLive() };
		def votes = [];
		lives.each {
			votes << new Vote(it);
		}
		// Tallies up voting results.
		lives.each {
			def target = it.target;
			votes.find{ target == it.actor }.number++;
		}
		// Sort the results.
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		votes.sort { one, two ->
			if(one.number == two.number) {
				return collator.compare(one.actor.name, two.actor.name);
			} else {
				return two.number <=> one.number;
			}
		};
		if(votes[0].number == votes[1].number) {
			revoteCount++;
			if(0 < revotelimit && revotelimit < revoteCount) {
				return true;
			} else {
				def clipboard = new StringBuilder(30 * votes.size());
				votes.each {
					noticeall(strings.getString("notice.vote.result", it.actor.name, it.number, it.actor.target.name));
					clipboard << strings.getString("notice.vote.result", it.actor.name, it.number, it.actor.target.name) << "\n";
				}
				if(0 < revotelimit) {
					noticeall(strings.getString("notice.vote.again.limited", revotelimit - revoteCount + 1));
					clipboard << strings.getString("notice.vote.again.limited", revotelimit - revoteCount + 1);
				} else {
					noticeall(strings.getString("notice.vote.again"));
					clipboard << strings.getString("notice.vote.again");
				}
				resetActors();
				JinroInterface.copyClipboard(clipboard.toString().denormalize());
				return false;
			}
		} else {
			revoteCount = 0;
			return true;
		}
	}
	// Execution
	def execution() {
		def votes = [];
		def lives = actors.findAll { it.isLive() };
		lives.each {
			votes << new Vote(it);
		}
		// Tallies up voting results.
		lives.each {
			def target = it.target;
			votes.findAll{ it.actor == target }.each {
				it.number++;
			}
		}
		// Sort the results.
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		votes.sort { one, two ->
			if(one.number == two.number) {
				return collator.compare(one.actor.name, two.actor.name);
			} else {
				return two.number <=> one.number;
			}
		};
		def clipboard = new StringBuilder(30 * votes.size());
		votes.each {
			noticeall(strings.getString("notice.vote.result", it.actor.name, it.number, it.actor.target.name));
			clipboard << strings.getString("notice.vote.result", it.actor.name, it.number, it.actor.target.name) << "\n";
		}
		if(votes[0].number == votes[1].number) {
			noticeall(strings.getString("notice.drawn"));
			clipboard << strings.getString("notice.drawn");
		} else {
			votes[0].actor.killed(State.executed);
			noticeall(strings.getString("notice.executed", votes[0].actor.name));
			clipboard << strings.getString("notice.executed", votes[0].actor.name);
			lives.each {
				if(it.target.name == votes[0].actor.name) {
					it.sanity -= sanPenaltyExecution;
				} else {
					it.sanity += sanRecoveryExecution;
				}
			}
		}
		JinroInterface.copyClipboard(clipboard.toString().denormalize());
	}
	// Judges victory or defeat.
	def judge() {
		def wolves = actors.findAll { it.isWolf() && it.isLive() };
		def civilians = actors.findAll { it.isCivilian() && it.isLive() };
		def foxes = actors.findAll { (it.roleIs(new Fox()) || it.roleIs(new Minifox())) && it.isLive() };
		def result = 0;
		if(0 < revotelimit && revotelimit < revoteCount) {
			noticeall(strings.getString("notice.judge.drawn"));
			result = -1;
			noticeResults(result);
			return true;
		} else if(!wolves) {
			if(foxes) {
				noticeall(strings.getString("notice.judge.winners", JinroGame.getAlias((new Fox()).getRoleName())));
				result = 2;
			} else {
				noticeall(strings.getString("notice.judge.winners", JinroGame.getAlias((new Civilian()).getRoleName())));
			}
			noticeResults(result);
			return true;
		} else {
			if(wolves.size() >= civilians.size()) {
				if(foxes) {
					noticeall(strings.getString("notice.judge.winners", JinroGame.getAlias((new Fox()).getRoleName())));
					result = 2;
				} else {
					noticeall(strings.getString("notice.judge.winners", JinroGame.getAlias((new Wolf()).getRoleName())));
					result = 1;
				}
				noticeResults(result);
				return true;
			}
			return false;
		}
	}
	// Notifies all actors the game results.
	def noticeResults(result) {
		def sortedactors = actors.clone();
		def clipboard = new StringBuilder(30 * sortedactors.size());
		def collator = Collator.getInstance(JinroLocalization.getLocale());
		collator.setStrength(Collator.PRIMARY);
		sortedactors.sort { one, two ->
			if(result < 0 || one.judge(result) == two.judge(result)) {
				if(one.getPriority() == two.getPriority()) {
					collator.compare(one.name, two.name);
				} else {
					return two.getPriority() <=> one.getPriority();
				}
			} else if(one.judge(result)) {
				return -1;
			} else {
				return 1;
			}
		};
		sortedactors.each { actor ->
			def text = strings.getString("notice.roles.result." + (result < 0 ? "draw" : actor.judge(result) ? "win" : "lose"));
			noticeall(strings.getString("notice.roles.result", actor.name, text, JinroGame.getAlias(actor.getRoleName())));
			clipboard << strings.getString("notice.roles.result", actor.name, text, JinroGame.getAlias(actor.getRoleName())) << "\n";
		}
		JinroInterface.copyClipboard(clipboard.toString().denormalize());
		// Shows the results for viewers.
		issecret = false;
	}
	// Goes to next phase.
	def update() {
		def lives = actors.findAll { it.isLive() };
		if(JinroGame.time == 0) {
			JinroGame.time++;
			if(!(0 < revotelimit && revotelimit < revoteCount)) {
				noticeall(strings.getString("notice.datetime", JinroGame.day, strings.getString("notice.datetime.night")));
			}
		} else {
			JinroGame.time--;
			JinroGame.day++;
			noticeall(strings.getString("notice.datetime", JinroGame.day, strings.getString("notice.datetime.morning")));
		}
		lives.size().times { i ->
			lives.size().times { j ->
				if((j - i) * lives[i].getUpdatePriority(lives[j]) < 0 ||
				   (i - j) * lives[j].getUpdatePriority(lives[i]) < 0) {
					def temp = lives[i];
					lives[i] = lives[j];
					lives[j] = temp;
				}
			};
		};
		lives.each { it.update() };
		def deadCount = killedMessages.size();
		lives.findAll{ it.isSanityEnabled() }.each { it.sanity += deadCount >= 2 ? sanPenaltyVictims : deadCount ? 0 : sanRecoveryVictims };
		Collections.shuffle(killedMessages);
		killedMessages.each { noticeall(it) };
		killedMessages.clear();
		resetActors();
	}
	def dealRoles() {
		killedMessages.clear();
		super.dealRoles();
		revoteCount = 0;
		actors.each {
			it.sanity = initialSanity;
			it.lunatic = false;
		};
	}
	def noticeKilled(message) {
		killedMessages << message;
	}
	def resetActors() {
		actors.each { it.reset() };
	}
	def Game() {
		JinroInterface.addColumn(strings.getString("sanity"), 60);
	}
}

class Vote {
	def actor;
	def number = 0;
	Vote(actor) {
		this.actor = actor;
	}
}
