package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Hunter
// Camp: Civilians

class Hunter extends Civilian {
	def static order = 5;
	def static strings = Civilian.strings.attachBundle("roles/Hunter.properties");
	def static number = 0;		// All roles must define own "number" property.
	def static bite = false;	// All roles must define own "bite" property.
	// Returns the proper number.
	def static getJustnumber(number) {
		if(number > 10) {
			return 1;
		} else {
			return 0;
		}
	}
	def getPriority() {
		return 40;
	}
	Hunter(actor = null) {
		super(actor);
	}
	def getRoleName() {
		return strings.getString("hunter");
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				notice(strings.getString("notice.defend"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(!target) {
					actors.findAll{ it.name != name && it.isLive() }.each {
						commands << strings.getString("command.defend") + "@" + it.name;
					};
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				if(type != strings.getString("command.defend")) {
					return;
				}
				if(this.target) {
					return;
				}
				if(target?.name != name && target?.isLive()) {
					this.target = target;
					notice(strings.getString("notice.defend.target", target.name));
					defend();
				}
				break;
		}

	}
	// Checks if the game can go to next phase.
	def checkStep() {
		if(!super.checkStep()) {
			return false;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 0:
				return true;
			case 1:
				if(JinroGame.day == 0) {
					return true;
				}
				if(target) {
					return true;
				}
				break;
		}
		return false;
	}
	def defend() {
		target.defended = true;
	}
}
