package scripts.roles;

import scripts.*;
import jinro.*;

// Role: Wolf (Werewolf)
// Camp: Wolves

class Wolf extends Civilian {
	def static order = 1;
	def static strings = Civilian.strings.attachBundle("roles/Wolf.properties");
	def static number = 0;	// All roles must define own "number" property.
	def static bite = true;	// All roles must define own "bite" property.
	def static isInvisible() {
		return true;
	}
	// Returns the proper number.
	def static getJustnumber(number) {
		if(number > 19) {
			return 4;
		} else if(number > 15) {
			return 3;
		} else if(number > 8) {
			return 2;
		} else {
			return 1;
		}
	}
	def getPriority() {
		return 100;
	}
	Wolf(actor = null) {
		super(actor);
	}
	def static setBitetarget(target) {
		JinroGame.setProperty("bitetarget", target);
	};
	def static getBitetarget() {
		return JinroGame.getProperty("bitetarget");
	}
	def getRoleName() {
		return strings.getString("wolf");
	}
	// Calls when start argument/night time.
	def startPhase() {
		super.startPhase();
		switch(JinroGame.time) {
			case 0:
				break;
			case 1:
				if(JinroGame.day == 0) {
					return;
				}
				notice(strings.getString("notice.bite"));
				break;
		}
	}
	// Returns the actor's command list
	def getCommands(actors) {
		def commands = super.getCommands(actors);
		if(!isLive()) {
			return commands;
		}
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(JinroGame.day == 0) {
					break;
				}
				if(!bitetarget) {
					actors.findAll{ !it.isWolf() && it.isLive() }.each {
						commands << strings.getString("command.bite") + "@" + it.name;
					};
				}
				break;
		}
		return commands;
	}
	def receiveMessage(type, target) {
		super.receiveMessage(type, target);
		// 0: daytime, 1: nighttime
		switch(JinroGame.time) {
			case 1:
				if(type != strings.getString("command.bite")) {
					return;
				}
				if(JinroGame.day == 0) {
					return;
				}
				if(bitetarget) {
					return;
				}
				if(!target?.isWolf() && target?.isLive()) {
					bitetarget = target;
					notice(strings.getString("notice.bite.target", target.name));
				}
				break;
		}
	}
	// Notifies the actor of the role.
	def noticeRole() {
		super.noticeRole();
		def actors = JinroGame.getActors();
		actors.findAll{ it.name != name && it.isWolf() }.each {
			notice(strings.getString("notice.partner", it.name, JinroGame.getAlias(new Wolf().getRoleName())));
		}
	}
	def isSanityEnabled() {
		return false;
	}
	// Judges victory or defeat.
	// if the result is... 0: civilians win, 1: wolves win, 2: foxes win
	def judge(result) {
		return result == 1;
	}
	// If it returns true, the role count as a wolf.
	def isWolf() {
		return true;
	}
	// If it returns true, the role count as a civilian.
	def isCivilian() {
		return false;
	}
}
